import numpy as np
import soundfile as sf
from scipy.linalg import toeplitz
from scipy.signal import welch
import matplotlib.pyplot as plt

# Read in audiofiles
wn_noise, samplerate = sf.read('lp_noise.wav')
colored, samplerate = sf.read('filtered.wav')
l = len(wn_noise)

# Plot frequency content
f, P_c = welch(colored, samplerate)
f, P_wn = welch(wn_noise, samplerate)

plt.semilogy(f,P_c)
plt.semilogy(f,P_wn)
plt.legend((['colored','white']))
plt.xlabel('frequency [Hz]')
plt.ylabel('Power in dB')
plt.show(block='True')

# Set length of prediction filter
N = 5

# Auto-correlation
corr_long = np.convolve(colored,np.flip(colored))
corr = corr_long[l-1:]

# Vector 
r = corr[1:N+1]

# Matrix
R = toeplitz(corr[0:N])

# Optimal filter coefficients (prediction)
hopt = np.matmul(np.linalg.inv(R),r)

# Prediction-error filter
h_e = np.append(1, -hopt)

# Error signal
error_long = np.convolve(colored,h_e)
error = error_long[0:l]

# Plot frequency content
f, P_e = welch(error,samplerate)

plt.semilogy(f,P_c)
plt.semilogy(f,P_wn)
plt.semilogy(f,P_e)
plt.legend((['colored','white','error']))
plt.xlabel('frequency [Hz]')
plt.ylabel('Power in dB')
plt.show(block='True')
