import numpy as np
import soundfile as sf
import matplotlib.pyplot as plt
from matplotlib.ticker import MaxNLocator

# Define DFT length
N_dft = 16

# Read in LPC
coeff, samplerate = sf.read('h_coeff.wav')
l = len(coeff)

# Plot LPC 
ax = plt.figure().gca()
ax.stem(range(0,l),coeff,use_line_collection=True)
plt.xlabel('coefficient index')
plt.ylabel('value')
plt.title('LPC')
ax.xaxis.set_major_locator(MaxNLocator(integer=True))
plt.grid()
plt.show(block=True)

#########################
# LPCC using DFT method #
#########################

# Create pred-error filter
pe = np.array(1)
pe = np.append(pe,-1*coeff)

# DFT of pred-error filter
P = np.fft.fft(pe,N_dft)

# DFT of inverse pred-error filter
P_inv = 1/P

# Compare spectra
plt.plot(20*np.log10(np.abs(P[0:int(N_dft/2+1)])))
plt.plot(20*np.log10(np.abs(P_inv[0:int(N_dft/2+1)])))
plt.legend(['prediction-error filter','inverse prediction-error filter'])
plt.xlabel('frequency index')
plt.ylabel('magnitude spectrum [dB]')
plt.grid()
plt.xlim(0,N_dft/2)
plt.show(block=True)

# To cepstral domain
P_inv_log = np.log(P_inv)
c_dft = np.fft.ifft(P_inv_log)

########################
# LPCC using recursion #
########################

# Preparation
coeff = np.append(coeff,np.zeros((N_dft-l,)))
c_rec = np.zeros((N_dft,))

# Iterate over LPCC index
for i in range(N_dft):
    if i < 1:
        c_rec[i] = 0
    elif 1 <= i <= l:
        # Recursion
        for k in range(1,i):
            c_rec[i] = c_rec[i] + c_rec[k]*k*coeff[i-k-1]
        c_rec[i] = c_rec[i]/i + coeff[i-1]
    else:
        # Recursion
        for k in range(1,i):
            c_rec[i] = c_rec[i] + c_rec[k]*k*coeff[i-k-1]
        c_rec[i] = c_rec[i]/i

###################
# Compare methods #
###################

# Plot weights
plt.stem(np.abs(c_dft),linefmt='r',markerfmt='ro',use_line_collection=True,basefmt='grey',label='DFT method')
plt.plot(np.abs(c_rec),color='b',label='Recursion')
plt.title('LPCC')
plt.legend()
plt.grid()
plt.xlim(-0.5,N_dft-0.5)
plt.show(block=True)
